﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_find_primes
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Find a prime.
        private Random Rand = new Random();
        private void btnFindPrime_Click(object sender, EventArgs e)
        {
            txtResult.Clear();
            Refresh();

            // Get the bounds.
            int min = int.Parse(txtMinimum.Text);
            int max = int.Parse(txtMaximum.Text);
            int num_tests = (int)nudNumTests.Value;

            // Find a prime.
            int p = FindPrime(min, max, num_tests);
            Console.WriteLine("Prime: " + p);
        }

        // Probabilistically find a prime number within the range [min, max].
        private int FindPrime(int min, int max, int num_tests)
        {
            // Try random numbers until we find a prime.
            for (int i = 1; ; i++)
            {
                // Pick a random odd p.
                int p = Rand.Next(min, max + 1);
                if (p % 2 == 0) continue;
                txtResult.Text = "Checking " + p;
                txtResult.Refresh();

                // See if it's prime.
                if (IsProbablyPrime(p, 100))
                {
                    txtResult.Text = p.ToString() + " (" + i + " tries)";
                    return p;
                }
            }
        }

        // Perform tests to see if a number is (probably) prime.
        private bool IsProbablyPrime(int p, int num_tests)
        {
            checked
            {
                // Perform the tests.
                for (int i = 0; i < num_tests; i++)
                {
                    // Pick a number n in the range (1, p).
                    long n = Rand.Next(2, p);

                    // Calculate n ^ (p - 1).
                    long result = n;
                    for (int power = 1; power < p - 1; power++)
                    {
                        result = (result * n) % p;
                    }

                    // If the final result is not 1, p is not prime.
                    if (result != 1) return false;
                }
            }

            // If we survived all the tests, p is probably prime.
            return true;
        }
    }
}
